//+------------------------------------------------------------------+
//|                                          optimal tracking filter |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_chart_window
#property indicator_buffers 3
#property indicator_color1  DeepSkyBlue
#property indicator_color2  PaleVioletRed
#property indicator_color3  PaleVioletRed
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  2

//
//
//
//
//

extern int  Price            = PRICE_MEDIAN;
extern int  PriceToCompareTo = PRICE_CLOSE;
extern bool alertsOn         = false;
extern bool alertsOnCurrent  = false;
extern bool alertsMessage    = true;
extern bool alertsSound      = false;
extern bool alertsEmail      = false;

//
//
//
//
//

double otf[];
double otfDa[];
double otfDb[];
double trend[];
double buff[][3];

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(4);
   SetIndexBuffer(0,otf);
   SetIndexBuffer(1,otfDa);
   SetIndexBuffer(2,otfDb);
   SetIndexBuffer(3,trend);

   //
   //
   //
   //
   //
   
   string PriceType;
      switch(Price)
      {
         case PRICE_CLOSE:    PriceType = "Close";    break;  // 0
         case PRICE_OPEN:     PriceType = "Open";     break;  // 1
         case PRICE_HIGH:     PriceType = "High";     break;  // 2
         case PRICE_LOW:      PriceType = "Low";      break;  // 3
         case PRICE_MEDIAN:   PriceType = "Median";   break;  // 4
         case PRICE_TYPICAL:  PriceType = "Typical";  break;  // 5
         case PRICE_WEIGHTED: PriceType = "Weighted"; break;  // 6
      }      

   //
   //
   //
   //
   //

   IndicatorShortName (" Optimal tracking filter ("+PriceType+")");
   return(0);
}
int deinit()
{
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

#define _prc  0
#define _tmp1 1
#define _tmp2 2

int start()
{
   int counted_bars=IndicatorCounted();
   int i,r,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         if (ArrayRange(buff,0) != Bars) ArrayResize(buff,Bars);

   //
   //
   //
   //
   //
   //

   if (trend[limit]==-1) CleanPoint(limit,otfDa,otfDb);
   for(i=limit, r=Bars-i-1; i >= 0; i--,r++)
   {
      buff[r][_prc] = iMA(NULL,0,1,0,MODE_SMA,Price,i);
      
      //
      //
      //
      //
      //

         buff[r][_tmp1] = 0.2*(buff[r][_prc]-buff[r-1][_prc])+0.8*buff[r-1][_tmp1];
         buff[r][_tmp2] = 0.1*(High[i]-Low[i])               +0.8*buff[r-1][_tmp2];
         if (buff[r][_tmp2]!=0)
               double lambda = MathAbs(buff[r][_tmp1]/buff[r][_tmp2]);
         else         lambda = 0;
         double alpha = ( -lambda*lambda + MathSqrt(lambda*lambda*lambda*lambda + 16.0*lambda*lambda))/8.0;

         //
         //
         //
         //
         //

         trend[i] = trend[i+1];
         otfDa[i] = EMPTY_VALUE;
         otfDb[i] = EMPTY_VALUE;
         otf[i] = otf[i+1]+alpha*(buff[r][_prc]-otf[i+1]);
         double price = iMA(NULL,0,1,0,MODE_SMA,PriceToCompareTo,i);
            if (price > otf[i]) trend[i] =  1;
            if (price < otf[i]) trend[i] = -1;
            if (trend[i]==-1) PlotPoint(i,otfDa,otfDb,otf);
   }
   manageAlerts();
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

//
//
//
//
//

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (first[i+1] == EMPTY_VALUE)
      {
         if (first[i+2] == EMPTY_VALUE) {
                first[i]   = from[i];
                first[i+1] = from[i+1];
                second[i]  = EMPTY_VALUE;
            }
         else {
                second[i]   =  from[i];
                second[i+1] =  from[i+1];
                first[i]    = EMPTY_VALUE;
            }
      }
   else
      {
         first[i]  = from[i];
         second[i] = EMPTY_VALUE;
      }
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; 
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] == 1) doAlert(whichBar,"abowe");
         if (trend[whichBar] ==-1) doAlert(whichBar,"bellow");
      }         
   }
}   

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   string addition = "";
      if (!alertsOnCurrent) addition="closed ";
   
      if (previousAlert != doWhat || previousTime != Time[forBar]) {
          previousAlert  = doWhat;
          previousTime   = Time[forBar];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," price "+addition+doWhat+" optimal tracking filter");
             if (alertsMessage) Alert(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol(),"otf "),message);
             if (alertsSound)   PlaySound("alert2.wav");
      }
}


